#include "lmsg/MsgHeader.hh"
#include "lmsg/Grinder.hh"
#include <iostream>

lmsg::MsgHeader::~MsgHeader(void) 
{}

lmsg::MsgHeader::MsgHeader(void) 
  : mLength(0), mType(0), mTransID(0), mBlockID(0) 
{}

lmsg::MsgHeader::MsgHeader(MsgType type, count_t length, const MsgAddr& dest)
  : mLength(length), mType(type), mDest(dest), mTransID(0), mBlockID(0)
{}

lmsg::MsgHeader::MsgHeader(const MsgHeader& h) {
    *this = h;
}

lmsg::MsgHeader&
lmsg::MsgHeader::operator=(const MsgHeader& h) {
    mLength  = h.mLength;
    mType    = h.mType;
    mDest    = h.mDest;
    mSource  = h.mSource;
    mTransID = h.mTransID;
    mBlockID = h.mBlockID;
    return *this;
}

void
lmsg::MsgHeader::Dump(std::ostream& out) const {
    out << "Message Header:" << std::endl;
    out << "  Data Length         " << getMsgLength() << std::endl;
    out << "  Message Type        " << getMsgType() << std::endl;
    out << "  Source Address      " << getSource() << std::endl;
    out << "  Destination Address " << getDest() << std::endl;
    out << "  Transaction ID      " << getTransID() << std::endl;
    out << "  Block ID            " << getBlockID() << std::endl;
}

void
lmsg::MsgHeader::Export(void) {
    Import();
}

static lmsg::Grinder g;

void
lmsg::MsgHeader::Import(void) {
    char*       Out = (char*) &mLength;
    const char* In  = Out;
    g.swap4(&In,  &Out,  2);

    Out = (char*) &mTransID;
    In  = (char*) Out;
    g.swap4(&In,  &Out,  2);
}

void 
lmsg::MsgHeader::setDest(const MsgAddr& addr) {
    mDest = addr;
}

void 
lmsg::MsgHeader::setSource(const MsgAddr& addr) {
    mSource = addr;
}

void 
lmsg::MsgHeader::setMsgLength(count_t length) {
    mLength = length;
}

void 
lmsg::MsgHeader::setMsgType(MsgType type) {
    mType = type;
}

void 
lmsg::MsgHeader::setTransID(count_t id) {
    mTransID = id;
}

void 
lmsg::MsgHeader::setBlockID(count_t id) {
    mBlockID = id;
}
