#include "lmsg/MsgAddr.hh"
//
//   The Solaris 8 header files seem to generate inconsistent code if
//   _POSIX_C_SOURCE is defined!
#if defined(sun) && defined(_POSIX_C_SOURCE)
#undef _POSIX_C_SOURCE
#endif
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string>
#include <iostream>

using namespace lmsg;
using namespace std;

MsgAddr 
MsgAddr::findIPHost(const char* HostID) {
    string Host;
    MsgAddr::ipport_t port=0;
    MsgAddr::ipport_t sbid=0;

    //----------------------------------  Get the port number if specified
    bool xPort(false), xSbID(false);
    for (const char* p=HostID ; *p ; p++) {
        if (*p == ':') {
	    if (xPort || xSbID) break;
	    xPort = true;
	} else if (*p == '[') {
	    if (xSbID) break;
	    xSbID = true;
	} else if (*p == ']') {
	    break;
	} else if (xSbID) {
	    if (*p < '0' || *p > '9') break;
	    sbid *= 10;
	    sbid += *p - '0';
	} else if (xPort) {
	    if (*p < '0' || *p > '9') break;
	    port *= 10;
	    port += *p - '0';
	} else {
	    Host += *p;
	}
    }

    //----------------------------------  Get the IP address
    MsgAddr::ipaddr_t addr=0;
    if (!Host.empty()) {
        struct hostent* ptr = gethostbyname(Host.c_str());
        if (ptr) addr = ntohl(*(ipaddr_t *) ptr->h_addr_list[0]);
    }
    return MsgAddr(addr, port, sbid);
}

static char retline[65];

const char* 
MsgAddr::getIPName(void) const 
{
    //----------------------------------  Get the IP address
    ipsock_t ipaddr;
    getAddr(ipaddr);

    //----------------------------------  Get the IP name
    const char* adptr = (char*)&ipaddr.sin_addr;
    struct hostent* ptr = gethostbyaddr(adptr, 
					sizeof(ipaddr.sin_addr), 
					ipaddr.sin_family);
    if (!ptr || !ptr->h_name) {
        sprintf(retline, "%d.%d.%d.%d", 
		adptr[0], adptr[1], adptr[2], adptr[3]);
        return retline;
    } else {
        return ptr->h_name;
    }
}


ostream& 
operator<<(ostream& out, const MsgAddr& addr) {
    return out << addr.getIPName() << ":" << addr.getIPPort() 
	       << "[" << addr.getSubProcess() << "]";
}
