#include "lmsg/Buffer.hh"
#include "lmsg/TransInput.hh"
#include "lmsg/Grinder.hh"
#include "Time.hh"
#include <string>

using namespace std;
using namespace lmsg;

static Grinder grub;

TransInput::TransInput(const Buffer& b) {
    mData   = b.getDataAddr();
    mIndex  = 0;
    mLength = b.getDataLength();
}

TransInput::TransInput(void) 
  : mData(0), mIndex(0), mLength(0)
{
}

lmsg::size_type 
TransInput::read(char* x, size_type N) throw(runtime_error) {
    align(sizeof(char));
    if (mIndex + N > mLength) throw underflow_error("End-Of-Data");
    memcpy(x, mData+mIndex, N);
    mIndex += N;
    return N;
}

lmsg::size_type 
TransInput::read(string* x, size_type N) throw(runtime_error) {
    if (!N) return 0;
    short Ls;
    for (size_type i=0 ; i< N ; i++) {
        read(&Ls);
	if (mIndex+Ls > mLength) throw underflow_error("End-Of-Data");
	*x++ = string(mData+mIndex, Ls);
	mIndex += Ls;
    }
    return N;
}

lmsg::size_type 
TransInput::read(short* x, size_type N) throw(runtime_error) {
    align(sizeof(short));
    if (!N) return 0;
    size_type lBytes = N*sizeof(short);
    if (mIndex+lBytes > mLength) throw underflow_error("End-Of-Data");
    const char* in(mData+mIndex);
    grub.swap2(&in, (char**)&x, N);
    mIndex += lBytes;
    return N;
}

lmsg::size_type 
TransInput::read(int* x, size_type N) throw(runtime_error) {
    align(sizeof(int));
    if (!N) return 0;
    size_type lBytes = N*sizeof(int);
    if (mIndex+lBytes > mLength) throw underflow_error("End-Of-Data");
    const char* in(mData+mIndex);
    grub.swap4(&in, (char**)&x, N);
    mIndex += lBytes;
    return N;
}

lmsg::size_type 
TransInput::read(long* x, size_type N) throw(runtime_error) {
    align(sizeof(long));
    if (!N) return 0;
    size_type lBytes = N*sizeof(long);
    if (mIndex+lBytes > mLength) throw underflow_error("End-Of-Data");
    const char* in(mData+mIndex);
    grub.swap4(&in, (char**)&x, N);
    mIndex += lBytes;
    return N;
}

lmsg::size_type 
TransInput::read(float* x, size_type N) throw(runtime_error) {
    align(sizeof(float));
    if (!N) return 0;
    size_type lBytes = N*sizeof(float);
    if (mIndex+lBytes > mLength) throw underflow_error("End-Of-Data");
    const char* in(mData+mIndex);
    grub.swap4(&in, (char**)&x, N);
    mIndex += lBytes;
    return N;
}

lmsg::size_type 
TransInput::read(double* x, size_type N) throw(runtime_error) {
    align(sizeof(double));
    if (!N) return 0;
    size_type lBytes = N*sizeof(double);
    if (mIndex+lBytes > mLength) throw underflow_error("End-Of-Data");
    const char* in(mData+mIndex);
    grub.swap8(&in, (char**)&x, N);
    mIndex += lBytes;
    return N;
}

lmsg::size_type 
TransInput::read(Time* x, size_type N) throw(runtime_error) {
    align(sizeof(Time::ulong_t));
    if (!N) return 0;
    size_type lBytes = 2*N*sizeof(Time::ulong_t);
    if (mIndex+lBytes > mLength) throw underflow_error("End-Of-Data");
    const char* in(mData+mIndex);
    grub.swap4(&in, (char**)&x, 2*N);
    mIndex += lBytes;
    return N;
}

void
TransInput::align(size_type N) {
    size_type nByte = (-mIndex) & (N-1);
    mIndex += nByte;
}
