//
//   Shared memory stream class
//
#include <time.h>
#include "oSMbuf.hh"
#include "lsmp_prod.hh"
using namespace std;

//
//   Constructors.
oSMbuf::oSMbuf() {
    mProducer = (LSMP_PROD*) 0;
    mBuffer   = (char*) 0;
    mLength   = 0;
}

#if defined(__GNU_STDC_OLD)
oSMbuf::oSMbuf(const char *partition, ios::open_mode mode) {
#else
oSMbuf::oSMbuf(const char *partition, ios::openmode mode) {
#endif
    open(partition, mode);
    mBuffer   = (char*) 0;
    mLength   = 0;
}

oSMbuf*
#if defined(__GNU_STDC_OLD)
oSMbuf::open(const char *partition, ios::open_mode mode) {
#else
oSMbuf::open(const char *partition, ios::openmode mode) {
#endif
    if ((mode & (ios::in | ios::trunc))) return (oSMbuf*) 0;

    //---------------------------------  Create/open a new partition.
    mProducer = new LSMP_PROD(partition);
    if (!mProducer->valid()) return (oSMbuf*)0;
    return this;
}

//
//    Destructor
//
oSMbuf::~oSMbuf() {
    if (mProducer) {
	sync();
        delete mProducer;
    }
}

//
//    Allocate a buffer
//
int oSMbuf::doallocate() {
    //---------------------------------  Make sure a partition is attached.
    if (!mProducer)          return EOF;
    if (!mProducer->valid()) return EOF;

    //---------------------------------  Release a buffer if one is allocated.
    if (mBuffer) {
        mProducer->release(0);
	mBuffer = (char*) 0;
    }

    //---------------------------------  Get a new SM buffer.
    mBuffer = mProducer->get_buffer();
    mLength = mProducer->getBufferLength();
    setb(mBuffer, mBuffer+mLength, 1);
    return 1;
}

//
//    Release the current buffer for consumption.
//
int oSMbuf::overflow(int c) {

    //---------------------------------  Make sure a partition is attached.
    if (!mProducer)          return EOF;
    if (!mProducer->valid()) return EOF;

    //---------------------------------  Release a buffer if one is allocated.
    sync();

    //---------------------------------  Get a new SM buffer.
    if (!mBuffer) {
        mBuffer = mProducer->get_buffer();
	mLength = mProducer->getBufferLength();
    }
    setp(mBuffer, mBuffer+mLength);
    if (c != EOF) sputc(c);
    return 1;
}

//
//    Synchronize the buffer (flush the output)
//
int oSMbuf::sync(void) {
    int Length = out_waiting();
    if (mBuffer && (Length > 0)) {
        mProducer->release(Length);
	mBuffer = (char*) 0;
	mLength = 0;
	setp(mBuffer, mBuffer);
    }
    return 0;
}

//
//    Seek (but ye shant find)
//
#ifdef __GNU_STDC_OLD
std::streampos oSMbuf::seekoff(std::streamoff off, std::ios::seek_dir dir, 
		               std::ios::open_mode mode)
#else			   
std::streampos oSMbuf::seekoff(std::streamoff off, std::ios::seekdir dir, 
                               std::ios::openmode mode)
#endif
{
    if ((mode & ios::in) != 0) return EOF;
    if (dir != ios::cur) return EOF;
    int inx = (int) (pptr() - mBuffer) + off;
    if ((inx < 0) || (inx >= mLength)) return EOF;
    setp(mBuffer + inx, mBuffer + mLength);
    return inx;
}
