// this class manipulates data for output into a text file
//
// Edward Daw, 8th September 2000

#include <stdio.h>
#include "logicals.h"
#include "Time.hh"
#include "Rms_output.hh"

Rms_output::
Rms_output( char* file_name, int no_of_columns )
{
  // set the number of data columns
  total_no_of_columns = no_of_columns;

  // set the maximum length of a text entry in bytes
  //This was changed by KSG on 09/29/05 to 50 bytes
  entry_length = 30;
  //entry_length = 50;

  // set the size of the character arrays to store the
  // column contents
  header_line_1_set = new char [ entry_length *
			       no_of_columns ];
  header_line_2_set = new char [ entry_length *
			       no_of_columns ];
  data_array = new float [ no_of_columns ];
  samp_rate_array = new int [ no_of_columns ];

  // set file name
  sprintf(output_file_name,"%s",file_name);

  // enable writing of text to the file
  _make_text_file = true;

  header_finished = NO;
  current_column_number = 0;
  lines_of_data_written = 0;
  return;
}

void
Rms_output::
disableTextOutput(void) {
  _make_text_file = false;
  return;
}

void
Rms_output::
enableTextOutput(void) {
  _make_text_file = true;
  return;
}

int 
Rms_output::
update_samp_rate( int column, int value)
{
  if(samp_rate_array == NULL) {
    return -1;
  } else if ( (column < 0) || (column > total_no_of_columns) ) {
    return -2;
  }
  else {
    samp_rate_array[ column ] = value;
    return 0;
  }
}

int
Rms_output::
add_header( char* first_header_line_entry,
	    char* second_header_line_entry,
	    int column_number )
{
  // check that the header isn't already written
  if(header_finished == YES) {
    printf("Error: header already finished.\n");
    return ERROR;
  }
  
  // add column data, as long column_number matches the internal
  // counter, current_column_number
  if( column_number == current_column_number ) {
    sprintf( ( header_line_1_set + column_number * entry_length ),
	     "%s", first_header_line_entry );
    sprintf( ( header_line_2_set + column_number * entry_length ),
	     "%s", second_header_line_entry );
    ++current_column_number;
  }
  else {
    printf("Error: column number mismach.\n");
    return ERROR;
  }

  // if we have reached the last column, append the file with the
  // header.
  if( current_column_number == total_no_of_columns ) {
 
    if( _make_text_file ) {
      
      if( ( out = fopen( output_file_name , "a" ) )== NULL ) {
	printf("Error opening %s. Exiting.\n",output_file_name);
	return ERROR;
      }
      
      int header_count;
      //    fprintf(out,"#GPS\t");
      //The following was added by KSG on 9/29/05
      for( header_count = 0;
	   header_count < (total_no_of_columns);
	   header_count++ ) {
	//9/29/05 split header_line_1_set into substrings
	//string line_1_A(header_line_1_set.substr(0,3));
	//string line_1_B(header_line_1_set.substr(3,27));
	//header_line_1_set = line_1_A + "DMT-BRMS_" + line_1_B;
	// KSG 9/02/05 The following line is commented out and then modified in order
	// to see if we have correctly located the fprintf statement that prints out the channel names
	// KSG 9/16/05 This statement (that prints to the .dat file not the .gwf file) changed again
	// KSG hopefully final change on 9/29/05
	//fprintf(out,"# %s : %s\n",
	//fprintf(out,"#DMT-BRMS_	%s : %s\n",
	fprintf(out,"#H0:DMT-BRMS_%s:%s\n",
	//fprintf(out,"%s:%s\n",	
		(header_line_1_set + header_count * entry_length),
		(header_line_2_set + header_count * entry_length) );
      }
      fclose(out);

    } // if _make_text_file

    header_finished = YES;
    current_column_number = 0;
  }

  return OK;
}

int
Rms_output::
add_data( float value, int column_number )
{
  // check that I'm not still writing the header.
  if(header_finished == NO) {
    printf("Error: still writing header during data write attempt.\n");
    return ERROR;
  }

  // if the data column on which a write was attempted is the same
  // as the internal data column count, then load the data into the
  // data array.
  if( column_number == current_column_number ) {
    data_array[ column_number ] = value;
    ++ current_column_number;
  }
  // if the row of data is complete and the _make_text_file bit is
  // set to true, append the data file. Otherwise, reset the values of
  // the data columns to zero but don't write them to a text file.

  if( current_column_number == total_no_of_columns ) {

    if ( _make_text_file ) {
      
      if( ( out = fopen( output_file_name , "a" ) )== NULL ) {
	printf("Error opening %s. Exiting.\n",output_file_name);
	return ERROR;
      }
      fprintf(out,"%ld\t",gps_time_for_entry);
      for( int data_count = 0;
	   data_count < (total_no_of_columns-1);
	   data_count++ ) {
	fprintf(out,"%.4e\t",data_array[ data_count ] );
	data_array[ data_count ] = 0;
      }
      fprintf(out,"%.4e\n",data_array[ total_no_of_columns - 1 ] );
      data_array[ total_no_of_columns - 1 ] = 0;
      fclose(out);

    } else {

      for( int data_count = 0;
	   data_count < total_no_of_columns ;
	   data_count++ ) {
	data_array[ data_count ] = 0;
      }

    }

    current_column_number = 0;
    ++lines_of_data_written;
  }

  return OK;
}

int
Rms_output::
log_error( char* error_string, Time gpstime )
{
  if( _make_text_file ) {
    
    if( (out=fopen(output_file_name,"a")) == NULL ) {
      return -1;
    }
    fprintf(out,"# %ld ERROR: %s\n",gpstime.getS(),error_string);
    fclose(out);

  } // if _make_text_file

  printf("Sent error to file %s.\n",output_file_name);
  return 0;
}

Rms_output::~Rms_output ( )
{
  delete [] header_line_1_set;
  delete [] header_line_2_set;
  delete [] data_array;
  delete [] samp_rate_array;
  return;
}



