#include "cell.hh"
#include <iomanip>
#include <sstream>

using namespace table;
using namespace std;

//======================================  Cell constructor
Cell::Cell(const string& str) 
  :  mType(kUnknown), mContent(str)
{
    setType();
}

//======================================  Append a string
void 
Cell::append(const string& x) {
    mContent += x;
}

Cell&
Cell::operator|=(const Cell& c) {
    mContent += c.mContent;
    return *this;
}

//======================================  Sum cells
Cell&
Cell::operator+=(const Cell& c) {
    if (getType() == kNumeric && c.getType() == kNumeric) {
        setNumeric(getNumeric() + c.getNumeric());
    } else {
        mContent += c.mContent;
    }
    return *this;
}

//======================================  Set the type
void
Cell::setType(void) {
    if (mContent.empty()) mType = kNull;
    else {
        char* p = const_cast<char*>(mContent.c_str());
	strtod(p, &p);
	if (*p) mType = kString;
	else    mType = kNumeric;
    }
}

//======================================  Cell data accessors
double 
Cell::getNumeric(void) const throw(runtime_error) {
    if (mType == kNull) return 0.0;
    if (mType != kNumeric) throw runtime_error("Data not numeric");
    const char* p = mContent.c_str(); 
    return strtod(const_cast<char*>(p), 0);
}

//======================================  Cell data accessors
void
Cell::setNumeric(double val) {
    long ival = long(val);
    ostringstream str;
    str << setprecision(16);
    if (double(ival) == val) str << ival;
    else                     str << val;
    mContent = str.str();
    mType    = kNumeric;
}
